﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Runtime.Serialization;

namespace Test0160804
{
    [ServiceContract]
    public interface IDemo
    {
        [OperationContract]
        string Hello(string name);
    }

    class MyService : IDemo
    {
        public string Hello(string name)
        {
            return $"Hello, {name}.";
        }
    }

    [DataContract(Namespace = "addr-info", Name = "addr_info")]
    public class AddressInfo
    {
        /// <summary>
        /// 省
        /// </summary>
        [DataMember(Name = "province")]
        public string Province { get; set; }

        /// <summary>
        /// 市
        /// </summary>
        [DataMember(Name = "city")]
        public string City { get; set; }

        /// <summary>
        /// 详细地址
        /// </summary>
        [DataMember(Name = "details")]
        public string DetailAddress { get; set; }

        /// <summary>
        /// 邮政编号
        /// </summary>
        [DataMember(Name = "zipcode")]
        public string ZipCode { get; set; }
    }

    class Program
    {
        static void Main(string[] args)
        {
            Uri endpointAddress = new Uri("http://localhost:2016/demo");
            using (ServiceHost host = new ServiceHost(typeof(MyService)))
            {
                // 添加终结点，必须在Open之前完成。
                // Open之后修改就不会生效了。

                // 1、准备地址头
                AddressInfo addrinfo = new AddressInfo
                {
                    Province = "广东省",
                    City = "东莞市",
                    ZipCode = "523xxx",
                    DetailAddress = "火星镇XX小区非人道主义研究大厦K008室"
                };
                // 创建Header
                AddressHeader hd = AddressHeader.CreateAddressHeader(addrinfo);

                // 2、创建EndpointAddress
                EndpointAddress epaddress = new EndpointAddress(endpointAddress, hd);

                // 3、创建终结点实例
                BasicHttpBinding binding = new BasicHttpBinding(BasicHttpSecurityMode.None);
                ServiceEndpoint svendp = new ServiceEndpoint(ContractDescription.GetContract(typeof(IDemo)), binding, epaddress);

                // 4、添加终结点到host中
                host.AddServiceEndpoint(svendp);

                // 5、打开服务
                try
                {
                    host.Open();
                    Console.WriteLine("服务已启动。");
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }

                /* 以下是客户端调用 */
                // 1、准备地址头
                AddressInfo cl_addrinfo = new AddressInfo();
                cl_addrinfo.Province = "广东省";
                cl_addrinfo.City = "广州市";
                cl_addrinfo.DetailAddress = "火星镇XX小区非人道主义研究大厦K008室";
                cl_addrinfo.ZipCode = "523xxx";
                AddressHeader hdclient = AddressHeader.CreateAddressHeader(cl_addrinfo);
                // 2、创建终结点地址，并附带地址头
                EndpointAddress clepaddr = new EndpointAddress(endpointAddress, hdclient);
                // 3、创建binding，必须与服务器的定义一致
                BasicHttpBinding clientbinding = new BasicHttpBinding(BasicHttpSecurityMode.None);
                // 4、创建通道工厂
                ChannelFactory<IDemo> factory = new ChannelFactory<IDemo>(clientbinding, clepaddr);
                // 创建通道，由于内部已实现，可以用协定直接作为通道对象
                IDemo channel = factory.CreateChannel();
                // 5、调用服务
                string response = channel.Hello("Jack");
                Console.WriteLine("调用结果：{0}", response);
                factory.Close();

                Console.Read();
            }
        }
    }
}
